#include "stdafx.h"

#include "JVMStarter.hpp"
#include "message.h"

#include "EventLog.hpp"
#include "ExceptionFilter.hpp"
#include "JavaVMCallback.hpp"

#include <assert.h>


namespace
{
	class JVMEventListenerImpl : public JVMEventListener
	{
	public:
		JVMEventListenerImpl( const std::string& v_serviceName, FILE* v_pJVMLog, EventLog& v_eventLog )
			: serviceName_( v_serviceName )
			, eventLog_( v_eventLog )
			, jvmLog_( v_pJVMLog )
		{
		}

		virtual ~JVMEventListenerImpl()
		{
		}

		virtual void notifyJVMStarted()
		{
			const char* mes[] = { serviceName_.c_str() };
			eventLog_.reportEvent(
				MSG_JAVAVM_STARTED,
				1,
				mes,
				EVENTLOG_INFORMATION_TYPE
				);
		}

		virtual void notifyJVMExited( int v_exitCode )
		{
			CHAR buf[ 64 ];
			wsprintf( buf, "%d", v_exitCode );
			const char* mes[] = { buf, serviceName_.c_str() };
			eventLog_.reportEvent(
				MSG_JAVAVM_EXITED,
				2,
				mes,
				EVENTLOG_INFORMATION_TYPE
				);
		}

		virtual void notifyJVMAborted()
		{
			const char* mes[] = { serviceName_.c_str() };
			eventLog_.reportEvent(
				MSG_JAVAVM_ABORTED,
				1,
				mes,
				EVENTLOG_ERROR_TYPE
				);
		}

		virtual void notifyException( const std::string& v_target )
		{
			const char* mes[] = { v_target.c_str() };
			eventLog_.reportEvent(
				MSG_JAVAVM_EXCEPTION,
				1,
				mes,
				EVENTLOG_ERROR_TYPE
				);
		}

		virtual int notifyJVMLog( FILE* v_fp, const char* v_format, va_list v_args )
		{
			if( jvmLog_ != NULL ) {
				return vfprintf( jvmLog_, v_format, v_args );
			}
			return 0;
		}

		virtual void notifyJVMThreadNormalyTerminated( const std::string& v_target )
		{
			const char* mes[] = { v_target.c_str() };
			eventLog_.reportEvent(
				MSG_JAVAVM_THREAD_NORMALY_TERMINATED,
				1,
				mes,
				EVENTLOG_INFORMATION_TYPE
				);
		}

		virtual void notifyJVMThreadAbnomalTerminated( const std::string& v_target, const std::string& v_reason )
		{
			const char* mes[] = { v_target.c_str(), v_reason.c_str() };
			eventLog_.reportEvent(
				MSG_JAVAVM_THREAD_ABNORMAL_TERMINATED,
				2,
				mes,
				EVENTLOG_ERROR_TYPE
				);
		}

		virtual void notifyLibraryLoadFailure( const std::string& v_dllPath )
		{
			const char* mes[] = { v_dllPath.c_str() };
			eventLog_.reportEvent(
				MSG_PRELOADLIB_FAILURE,
				1,
				mes,
				EVENTLOG_ERROR_TYPE
				);
		}

	private:
		std::string serviceName_;
		FILE* jvmLog_;
		EventLog& eventLog_;
	};

	std::string GetConfPath( const std::string& v_ext ) throw()
	{
		CHAR preloadConfPath[ MAX_PATH + 9 ];
		preloadConfPath[ GetModuleFileName( NULL, preloadConfPath, MAX_PATH ) ] = 0;
		LPSTR p = preloadConfPath;
		while(*p)p++;
		while( p > preloadConfPath && *p != '\\' ) {
			if( *p == '.' ) {
				strcpy( p, ".preload" );
				break;
			}
			p = CharPrev( preloadConfPath, p );
		}
		if( p == preloadConfPath || *p == '\\' ) {
			return "";
		}
		return std::string( preloadConfPath );
	}

}

int main(int argc, char* argv[])
{
	UnhandledExceptionHandler::setUnhandledExceptionHandler();

	const std::string serviceName( "JVMStarter" );

	EventLog eventLog( serviceName );
	eventLog.registerEventSource();
	eventLog.open();

	UnhandledExceptionHandler::setEventLog( &eventLog );

	JVMEventListenerImpl jvmEventListener( serviceName, stdout, eventLog );

	const std::string preloadConfPath( GetConfPath( ".preload" ) );

	JVMStarter jvmStarter( jvmEventListener, preloadConfPath );
	if( jvmStarter.preloadLibrary() ) {
		jvmStarter.execute( "JVMStarter", "jvmstarter.ini" );
	}

	return 0;
}

